'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var ramda = require('ramda');
var ramdaExtension = require('ramda-extension');
var namespaces = require('@redux-tools/namespaces');
var utils = require('@redux-tools/utils');
var invariant = _interopDefault(require('invariant'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var FUNCTION_KEY = '@@redux-tools/injectors/FUNCTION_KEY';

var createEntry = ramda.applySpec({
  key: ramda.nth(0),
  value: ramda.nth(1)
});
/**
 * Converts the input of `store.injectSomething()` or `store.ejectSomething()`
 * to an array of standalone entries.
 *
 * @param {Object|Function} injectables an object or function with injectables as values
 * @param {Object} props props to store in the entry, e.g. `namespace` or `feature`
 * @returns {Object[]} an array of entries
 */

var createEntries = function createEntries(injectables, _ref) {
  if (_ref === void 0) {
    _ref = {};
  }

  var _ref2 = _ref,
      feature = _ref2.feature,
      otherProps = _objectWithoutPropertiesLoose(_ref2, ["feature"]);

  var sanitizedProps = ramdaExtension.rejectNil(_extends({}, otherProps, {
    feature: feature || namespaces.DEFAULT_FEATURE
  }));
  var createEntriesFromObject = ramda.compose(ramda.map(ramda.merge(sanitizedProps)), ramda.map(createEntry), ramda.toPairs);

  var createEntriesFromFunction = function createEntriesFromFunction(injectable) {
    return [_extends({
      key: FUNCTION_KEY,
      value: injectable
    }, sanitizedProps)];
  };

  return ramda.ifElse(ramdaExtension.isFunction, createEntriesFromFunction, createEntriesFromObject)(injectables);
};

var enhanceStore = function enhanceStore(prevStore, config, _temp) {
  var _extends2;

  var _ref = _temp === void 0 ? {} : _temp,
      _ref$onEjected = _ref.onEjected,
      onEjected = _ref$onEjected === void 0 ? ramdaExtension.noop : _ref$onEjected,
      _ref$onInjected = _ref.onInjected,
      onInjected = _ref$onInjected === void 0 ? ramdaExtension.noop : _ref$onInjected;

  invariant(ramdaExtension.isObject(prevStore), 'You must pass a Redux store as the first argument to `enhanceStore()`');
  invariant(ramdaExtension.isObject(config), 'You must pass an injector config as the second argument to `enhanceStore()`');
  var injectMethodName = config.injectMethodName,
      ejectMethodName = config.ejectMethodName,
      getEntries = config.getEntries,
      setEntries = config.setEntries,
      type = config.type;
  var _prevStore$dispatch = prevStore.dispatch,
      dispatch = _prevStore$dispatch === void 0 ? ramdaExtension.noop : _prevStore$dispatch;
  var actionType = ramdaExtension.toScreamingSnakeCase(type);

  var inject = function inject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    ramda.forEach(function (entry) {
      return invariant( // NOTE: `isFunction` from ramda-extension returns `false` for `jest.fn()`.
      typeof entry.value === 'function', "Injecting " + type + ", but the value of " + JSON.stringify(entry) + " is not a function.");
    }, entries);
    var nextEntries = ramda.concat(getEntries(nextStore), entries);
    setEntries(nextEntries, nextStore);
    onInjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_INJECTED",
      payload: ramda.keys(injectables),
      meta: props
    });
  };

  var eject = function eject(injectables, props) {
    if (props === void 0) {
      props = {};
    }

    var entries = createEntries(injectables, props);
    var nextEntries = utils.withoutOnce(entries, getEntries(nextStore));
    setEntries(nextEntries, nextStore);
    onEjected({
      injectables: injectables,
      props: props,
      entries: entries
    });
    dispatch({
      type: "@redux-tools/" + actionType + "_EJECTED",
      payload: ramda.keys(injectables),
      meta: props
    });
  };

  var nextStore = _extends({}, prevStore, (_extends2 = {}, _extends2[injectMethodName] = inject, _extends2[ejectMethodName] = eject, _extends2));

  setEntries([], nextStore);
  return nextStore;
};

var makeConfig = function makeConfig(type) {
  invariant(type, 'The config type must be defined.');
  var injectMethodName = "inject" + ramdaExtension.toPascalCase(type);
  var ejectMethodName = "eject" + ramdaExtension.toPascalCase(type);
  var entriesPath = ['entries', type];
  return {
    type: type,
    getInject: ramda.prop(injectMethodName),
    getEject: ramda.prop(ejectMethodName),
    injectMethodName: injectMethodName,
    ejectMethodName: ejectMethodName,
    getEntries: ramda.o(ramdaExtension.defaultToEmptyArray, ramda.path(entriesPath)),
    // NOTE: We need to mutate the store and handle missing `store.entries` object.
    setEntries: function setEntries(entries, store) {
      return store.entries = ramda.assocPath(entriesPath, entries, store).entries;
    }
  };
};

exports.FUNCTION_KEY = FUNCTION_KEY;
exports.createEntries = createEntries;
exports.enhanceStore = enhanceStore;
exports.makeConfig = makeConfig;
